/*----------------------------------------------------------*
    krslib.h
    KRSプロトコル用API V1.1
                                       Last Edit '13 09/12
   Copyright (c) 2013 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _KRSLIB_H_INCLUDE
#define _KRSLIB_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif
#ifdef _WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

#ifndef _MSC_VER
#include <stdbool.h>
#endif
#include <stdint.h>
#include <stdint.h>

#define BROADCASTING_ID     (0xfe)

#ifndef _WIN32
  typedef int                             HANDLE;
  typedef pthread_mutex_t                 CRITICAL_SECTION;

  #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
  #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
  #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
  #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
  #define SetEvent(evt)                   pthread_cond_signal(evt)
  #define INVALID_int_VALUE               -1
  #define INVALID_HANDLE_VALUE            -1
#endif

// デバイスID
#ifdef _WIN32
  #ifdef _WIN64
  typedef uint64_t TDeviceID;
  #else
  typedef uint32_t TDeviceID;
  #endif
#else
  typedef uint32_t TDeviceID;
#endif

// コマンド
typedef enum {CMD_WRITE_POS = 4, CMD_READ_PARAM = 5, CMD_WRITW_PARAM = 6, CMD_ID = 7} TCommand;

// サブコマンド
#define SC_EEPROM         (0)
#define SC_STRETCH        (1)
#define SC_SPEED          (2)
#define SC_CURRENT        (3)
#define SC_TEMPTURE       (4)

#define SC_READ_ID        (0)
#define SC_WRITE_ID       (1)

// エラーフラグ
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID   (1 << 15)
#define ERR_INVALID_ID      (1 << 14)
#define ERR_DIFF_ID         (1 << 13)
#define ERR_ILLEGAL_SIZE    (1 << 12)
#define ERR_INVALID_PARAM   (1 << 11)
#define ERR_COMM            (1 << 10)
#define ERR_CHECKSUM        (1 << 9)
#define ERR_TIMEOUT         (1 << 8)
#define ERR_KRS_INST        (1 << 6)
#define ERR_KRS_OVERLOAD    (1 << 5)
#define ERR_KRS_CHECKSUM    (1 << 4)
#define ERR_KRS_RANGE       (1 << 3)
#define ERR_KRS_OVERHEAT    (1 << 2)
#define ERR_KRS_ANGLE       (1 << 1)
#define ERR_KRS_OVERVOLTAGE (1 << 0)
#define ERR_NON             (0)

// アラームステータス
typedef struct {
  uint8_t     id;
  TErrorCode  Status;
} TAlarmStatus;

#ifdef _WIN32
 #ifdef __MAKE_LIB__
  #define KRSAPIDLL __declspec(dllexport) WINAPI
 #else
  #define KRSAPIDLL __declspec(dllimport) WINAPI
 #endif
#else
 #define KRSAPIDLL
 #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// 静的なDLL呼び出し時は以下のプロトタイプ宣言を使用
TDeviceID KRSAPIDLL KRS_OpenPort (char *name, uint32_t baud);
bool KRSAPIDLL KRS_ClosePort (TDeviceID dvid);
void KRSAPIDLL KRS_SetEchoCancel (bool becho);
bool KRSAPIDLL KRS_SetBaudrate (TDeviceID dvid, uint32_t baud);
bool KRSAPIDLL KRS_Active (TDeviceID dvid);
void KRSAPIDLL KRS_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
double KRSAPIDLL GetQueryPerformanceCounter (void);
bool KRSAPIDLL KRS_TxPacket (TDeviceID dvid, uint8_t id, TCommand cmd, uint8_t *param, uint32_t len, TErrorCode *err);
bool KRSAPIDLL KRS_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
bool KRSAPIDLL KRS_ReadByteData(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, TErrorCode *errcode);
bool KRSAPIDLL KRS_WriteByteData(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t dat, TErrorCode *errcode);
bool KRSAPIDLL KRS_ReadWordData(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t *rdata, TErrorCode *errcode);
bool KRSAPIDLL KRS_WriteWordData(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t dat, TErrorCode *errcode);
bool KRSAPIDLL KRS_ReadBlockData (TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, uint32_t len, TErrorCode *errcode);
bool KRSAPIDLL KRS_WriteBlockData(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *dat, uint32_t len, TErrorCode *errcode);
bool KRSAPIDLL KRS_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *errcode);
bool KRSAPIDLL KRS_Ping(TDeviceID dvid, uint8_t id, TErrorCode *errcode);

#else
// 動的なDLL呼び出し時は以下の型宣言を使用
typedef TDeviceID (WINAPI *_KRS_OpenPort)(char *name, long baud);
typedef bool (WINAPI *_KRS_ClosePort)(TDeviceID dvid);
typedef void (WINAPI *_KRS_SetEchoCancel)(bool becho);
typedef bool (WINAPI *_KRS_SetBaudrate) (TDeviceID dvid, long baud);
typedef bool (WINAPI *_KRS_Active)(TDeviceID dvid);
typedef void (WINAPI *_KRS_SetTimeOutOffset)(TDeviceID dvid, uint32_t offsettime);
typedef bool (WINAPI *_KRS_TxPacket) (TDeviceID dvid, uint8_t id, TCommand cmd, uint8_t *param, uint32_t len, TErrorCode *err);
typedef bool (WINAPI *_KRS_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, TErrorCode *err);
typedef bool (WINAPI *_KRS_ReadByteData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_WriteByteData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t dat, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_ReadWordData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t *rdata, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_WriteWordData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint16_t dat, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_ReadBlockData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *rdata, uint32_t len, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_WriteBlockData)(TDeviceID dvid, uint8_t id, uint8_t adr, uint8_t *dat, uint32_t len, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *errcode);
typedef bool (WINAPI *_KRS_Ping)(TDeviceID dvid, uint8_t id, TErrorCode *errcode);

_KRS_OpenPort         KRS_OpenPort = NULL;
_KRS_ClosePort        KRS_ClosePort = NULL;
_KRS_SetEchoCancel    KRS_SetEchoCancel = NULL;
_KRS_SetBaudrate      KRS_SetBaudrate = NULL;
_KRS_Active           KRS_Active = NULL;
_KRS_SetTimeOutOffset KRS_SetTimeOutOffset = NULL;
_KRS_TxPacket         KRS_TxPacket = NULL;
_KRS_RxPacket         KRS_RxPacket = NULL;
_KRS_ReadByteData     KRS_ReadByteData = NULL;
_KRS_WriteByteData    KRS_WriteByteData = NULL;
_KRS_ReadWordData     KRS_ReadWordData = NULL;
_KRS_WriteWordData    KRS_WriteWordData = NULL;
_KRS_ReadBlockData    KRS_ReadBlockData = NULL;
_KRS_WriteBlockData   KRS_WriteBlockData = NULL;
_KRS_WriteSyncData    KRS_WriteSyncData = NULL;
_KRS_Ping             KRS_Ping = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
#ifdef _WIN64
#define DLLNAME   "krslib_x64.dll"
#else
#define DLLNAME   "krslib_x32.dll"
#endif
#else
#ifdef __x86_64__
#define DLLNAME   "krslib_x64.dll"
#else
#define DLLNAME   "krslib_x32.dll"
#endif
#endif

// DLLのロード
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  KRS_OpenPort       = (_KRS_OpenPort)GetProcAddress (hModule, "KRS_OpenPort");
  KRS_ClosePort      = (_KRS_ClosePort)GetProcAddress (hModule, "KRS_ClosePort");
  KRS_SetEchoCancel  = (_KRS_SetEchoCancel)GetProcAddress (hModule, "KRS_SetEchoCancel");
  KRS_SetBaudrate    = (_KRS_SetBaudrate)GetProcAddress (hModule, "KRS_SetBaudrate");
  KRS_Active         = (_KRS_Active)GetProcAddress (hModule, "KRS_Active");
  KRS_SetTimeOutOffset = (_KRS_SetTimeOutOffset)GetProcAddress (hModule, "KRS_SetTimeOutOffset");
  KRS_TxPacket       = (_KRS_TxPacket)GetProcAddress (hModule, "KRS_TxPacket");
  KRS_RxPacket       = (_KRS_RxPacket)GetProcAddress (hModule, "KRS_RxPacket");
  KRS_ReadByteData   = (_KRS_ReadByteData)GetProcAddress (hModule, "KRS_ReadByteData");
  KRS_WriteByteData  = (_KRS_WriteByteData)GetProcAddress (hModule, "KRS_WriteByteData");
  KRS_ReadWordData   = (_KRS_ReadWordData)GetProcAddress (hModule, "KRS_ReadWordData");
  KRS_WriteWordData  = (_KRS_WriteWordData)GetProcAddress (hModule, "KRS_WriteWordData");
  KRS_ReadBlockData  = (_KRS_ReadBlockData)GetProcAddress (hModule, "KRS_ReadBlockData");
  KRS_WriteBlockData = (_KRS_WriteBlockData)GetProcAddress (hModule, "KRS_WriteBlockData");
  KRS_WriteSyncData  = (_KRS_WriteSyncData)GetProcAddress (hModule, "KRS_WriteSyncData");
  KRS_Ping           = (_KRS_Ping)GetProcAddress (hModule, "KRS_Ping");

  return 
    (KRS_OpenPort         != NULL) &&
    (KRS_ClosePort        != NULL) &&
    (KRS_SetEchoCancel    != NULL) &&
    (KRS_SetBaudrate      != NULL) &&
    (KRS_Active           != NULL) &&
    (KRS_SetTimeOutOffset != NULL) &&
    (KRS_TxPacket         != NULL) &&
    (KRS_RxPacket         != NULL) &&
    (KRS_ReadByteData     != NULL) &&
    (KRS_WriteByteData    != NULL) &&
    (KRS_ReadWordData     != NULL) &&
    (KRS_WriteWordData    != NULL) &&
    (KRS_ReadBlockData    != NULL) &&
    (KRS_WriteBlockData   != NULL) &&
    (KRS_WriteSyncData    != NULL) &&
    (KRS_Ping             != NULL);
}

// DLLのアンロード
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    KRS_OpenPort         = NULL;
    KRS_ClosePort        = NULL;
    KRS_SetEchoCancel    = NULL;
    KRS_SetBaudrate      = NULL;
    KRS_Active           = NULL;
    KRS_SetTimeOutOffset = NULL;
    KRS_TxPacket         = NULL;
    KRS_RxPacket         = NULL;
    KRS_ReadByteData     = NULL;
    KRS_WriteByteData    = NULL;
    KRS_ReadWordData     = NULL;
    KRS_WriteWordData    = NULL;
    KRS_ReadBlockData    = NULL;
    KRS_WriteBlockData   = NULL;
    KRS_WriteSyncData    = NULL;
    KRS_Ping             = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_KRSLIB_H_INCLUDE
